#include "MustuxAudioDeviceMapper.hh"
#include "MustuxDebugger.hh"

#include <linux/soundcard.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <qstring.h>

// KEEP NUMPERIODS = 4 FOR A WHILE
// TRY TO AVOID THE UNDERRUN PROBLEM WITHOUT CHANGING THE NUMPERIODS
// I THINK THIS IS A DESIGN PROBLEM


namespace MustuxAudioDeviceMapper
        {

        int rate;
        int channels;
        int bitDepth;

#ifdef _ALSA
        snd_pcm_sframes_t       fragmentSizeInBlocks[MAX_BUSES];
        snd_pcm_t*              pcm_handle[MAX_BUSES];
        snd_pcm_hw_params_t*    hwparams[MAX_BUSES];
        snd_pcm_sframes_t       buffer_size[MAX_BUSES];
#else
        int fragmentSizeInBlocks[MAX_BUSES];
#endif
        int r;
        int dir;
        int fps;
        int mode;
        int nBuses;

        int period_time[MAX_BUSES];
        int numPeriods[MAX_BUSES];
        int buffer_time[MAX_BUSES];
        int fragmentSize[MAX_BUSES];
        int deviceId[MAX_BUSES];
        QString device[MAX_BUSES];
        QString busName[MAX_BUSES];
        bool validBus[MAX_BUSES];
        bool busy[MAX_BUSES];

        }

int MustuxAudioDeviceMapper::init(int pFps)
        {
        PENTER;
#ifdef _ALSA
        for (int i=0; i<MAX_BUSES;i++) busName[i]="";
        FILE* f = fopen("/proc/asound/cards","r");
        if (!f)
                {
                PERROR("Cannot open /proc/asound/cards for reading ( maybe busy or no read permission ? )");
                PEXIT;
                return -1;
                }

        char c;
        QString s="";
        while (!feof(f))
                {
                fread(&c,1,1,f);
                s=s+QChar(c);
                }
        fclose(f);
        bool flag=true;
        int p=0;
        int devcounter=0;
        while (flag)
                {
                QString s1, s2;
                int d1 = s.find("\n",p);
                if (d1>=0)
                        {
                        s1=s.mid(p,d1-p);
                        int p2 = s1.find("]:");
                        if (p2>=0)
                                {
                                s2 = s1.mid(p2+3);
                                s2 = s2.simplifyWhiteSpace();
                                busName[devcounter++]=s2;
                                }
                        p=d1+1;
                        }
                else
                        break;
                }
        nBuses=devcounter;
        for (int i=0; i<nBuses; i++)
                {
                QString sb; sb.setNum(i);
                QString s =
                device[i]="plughw:"+sb+",0";
                }

        for (int i=0; i<nBuses; i++)
                {
                PMESG("Found Alsa Device # %d device=%s",i, (const char*) device[i].latin1());
                }
#else
        nBuses=1;
        device[0] = "/dev/dsp";
        busName[0]="OSS";
        deviceId[0] = 0;
#endif

        set_fps(pFps);


#ifndef _ALSA
        PWARN("*** There is no ALSA available. Forcing OSS mode. ");
        PWARN("*** WARNING:  Latency time might become greater and routing features will be disabled !");
#endif


        // CHECK ALL BUSES
        int ro=0;
        int rc=0;
        for (int b=0; b<nBuses; b++)
                {
                PMESG("Checking BUS %d ...",b);
                validBus[b]=true;
                busy[b]=false;
                ro = open_device(b, PLAY, 44100, 2, 16);
                if (ro < 0 )
                        {
                        PWARN("Bus %d seems to be unavailable. Invalidating it...",b);
                        validBus[b]=false;
                        }
                else
                        {
                        rc = close_device(b);
                        if (rc <0)
                                {
                                PWARN("Bus %d seems to be unavailable. Invalidating it...",b);
                                validBus[b]=false;
                                }
                        }
                }

        return 1;
        PEXIT;
        }

void MustuxAudioDeviceMapper::set_fps(int pFps)
        {
        fps = pFps;
        for (int i = 0; i < nBuses; i++)
                {
                period_time[i] = 1000000 / fps; // period time in usecs
                numPeriods[i] = nBuses * 4;// This formula was raised when I had a problem with 2 sound cards.
                                           // for 1 sound card, 4 periods was fine, when
                                           // I added another sound card, I got a lot
                                           // of buffer underrun. Then I raise to 8
                                           // periods and it was fixed. So , for now
                                           // I put the numPeriods as a function of nBuses
                                           // I wonder this is really not OK..
                                           // If I get 10 devices, I will have a very big num of periods...
                buffer_time[i] = period_time[i] * numPeriods[i];
                }
        }



int MustuxAudioDeviceMapper::open_device(int bus, int pMode, int pRate, int pChannels, int pBitDepth)
        {
        PENTER;
        if (!validBus[bus])
                {
                PERROR("Bus %d has been marked as unavailable. Cannot open it.",bus);
                PEXIT;
                return -1;
                }
        if (busy[bus])
                {
                PERROR("Bus %d is already in use.",bus);
                PEXIT;
                return -2;
                }

        mode=pMode;
        rate=pRate;
        channels=pChannels;
        bitDepth=pBitDepth;
        char* smode = (pMode==REC?strdup("REC"):strdup("PLAY"));

#ifndef _ALSA

        if (mode==PLAY)
                {
                deviceId[0] = open(device[0], O_WRONLY );
                if (deviceId[0] < 0)
                        {
                        PERROR("[OSS Handler] - Cannot open %s for playback . ( maybe busy ? )", (const char*) device[0].latin1());
                        PEXIT;
                        return -1;
                        }
                }
        else
                {
                deviceId[0] = open(device[0], O_RDONLY);
                if (deviceId[0] < 0)
                        {
                        PERROR("[OSS Handler] - Cannot open %s  for record. ( maybe busy ?)", (const char*) device[0].latin1());
                        PEXIT;
                        return -1;
                        }
                }
        static int arg , r;
        arg = channels;
        r = ioctl(deviceId[0], SOUND_PCM_WRITE_CHANNELS, &arg);
        if (r<0)
                {
                PERROR("[OSS Handler] - Cannot set number of channels");
                PEXIT;
                return -1;
                }
        arg = bitDepth;
        r = ioctl(deviceId[0], SOUND_PCM_WRITE_BITS, &arg);
        if (r<0)
                {
                PMESG("[OSS Handler] - Cannot set bit depth");
                PEXIT;
                return -1;
                }
        arg = rate;
        r = ioctl(deviceId[0], SOUND_PCM_WRITE_RATE, &arg);
        if (r<0)
                {
                PMESG("[OSS Handler] - Cannot set sample rate");
                PEXIT;
                return -1;
                }
        int fs;
        if (ioctl(deviceId[0], SNDCTL_DSP_GETBLKSIZE, &fs) == -1)
                {
                PERROR("[OSS Handler] - Cannot set optimal fragment size");
                PEXIT;
                return -1;
                }
        fragmentSize[0] = fs;
        fragmentSizeInBlocks[0] = fragmentSize[0] / (channels*(bitDepth/8));

        int format = AFMT_S16_LE;
        if (ioctl (deviceId[0], SNDCTL_DSP_SETFMT, &format) == -1)
                {
                PERROR("[OSS Handler] - Cannot set sample format");
                PEXIT;
                return -1;
                }

        PMESG("[OSS Handler] - BUS # %d Sucessfully open - REMEMBER THAT OSS SUPPORT IN MUSTUX IS DEPRECATED, SO DON'T EXPECT TO HAVE A STABLE BEHAVIOR",bus );
        PMESG("                DEV=%s MODE=%s ",(const char*) device[0].latin1(),smode);
        PMESG("                RATE=%d CHANNELS=%d BITDEPTH=%d",rate ,channels,bitDepth);
        PMESG("                FPS=%d  fragmentSizeInBlocks=%d  fragmentSize=%d",fps,(int)fragmentSizeInBlocks[0],(int)fragmentSize[0]);

#else
        const char* cdevice = (const char*) device[bus].latin1();

        snd_pcm_hw_params_alloca(&(hwparams[bus]));

        if (mode==PLAY)
                {
                // open the device for PLAY
                r = snd_pcm_open(&(pcm_handle[bus]), cdevice, SND_PCM_STREAM_PLAYBACK, 0);
                if ( r < 0)
                        {
                        PERROR("Playback open error: %s",snd_strerror(r));
                        PEXIT;
                        return r;
                        }
                }
        else if (mode==REC)
                {
                // open the device for PLAY
                r = snd_pcm_open(&(pcm_handle[bus]), cdevice, SND_PCM_STREAM_CAPTURE, 0);
                if (r < 0)
                        {
                        PERROR("Capture open error: %s",snd_strerror(r));
                        PEXIT;
                        return r;
                        }
                }


        // choose all parameters
        r = snd_pcm_hw_params_any(pcm_handle[bus], hwparams[bus]);
        if (r < 0)
                {
                PERROR("Broken configuration for playback: no configurations available: %s ",snd_strerror(r));
                PEXIT;
                return r;
                }

        // set the interleaved read/write format
        r = snd_pcm_hw_params_set_access(pcm_handle[bus], hwparams[bus], SND_PCM_ACCESS_RW_INTERLEAVED);
        if (r < 0)
                {
                PERROR("Access type not available for playback: %s",snd_strerror(r));
                PEXIT;
                return r;
                }

        // set the sample format
        r = snd_pcm_hw_params_set_format(pcm_handle[bus], hwparams[bus], SND_PCM_FORMAT_S16_LE);
        if (r < 0)
                {
                PERROR("Sample format not available for playback: %s",snd_strerror(r));
                PEXIT;
                return r;
                }

        // set the count of channels
        r = snd_pcm_hw_params_set_channels(pcm_handle[bus], hwparams[bus], channels);
        if (r < 0)
                {
                PERROR("Channels count %d not available for playbacks: %s",  channels, snd_strerror(r));
                PEXIT;
                return r;
                }

        // set the stream rate
        r = snd_pcm_hw_params_set_rate_near(pcm_handle[bus], hwparams[bus], rate, 0);
        if (r < 0)
                {
                PERROR("Rate %d  Hz not available for playback: %s",  rate, snd_strerror(r));
                PEXIT;
                return r;
                }
        if (r != rate)
                {
                PERROR("Warning : Rate doesn't match (requested %d Hz, allowed %d Hz)", rate,r );
                }

        // set buffer time
        r = snd_pcm_hw_params_set_buffer_time_near(pcm_handle[bus], hwparams[bus], buffer_time[bus], &dir);
        if (r < 0)
                {
                PERROR("Unable to set buffer time %d for playback: %s",buffer_time[bus],snd_strerror(r));
                PEXIT;
                return r;
                }
        buffer_size[bus] = snd_pcm_hw_params_get_buffer_size( hwparams[bus]);

        // set period time
        r = snd_pcm_hw_params_set_period_time_near(pcm_handle[bus], hwparams[bus], period_time[bus], &dir);
        if (r < 0)
                {
                PERROR("Unable to set period time %d  for playback: %s",period_time[bus],snd_strerror(r));
                PEXIT;
                return r;
                }
        fragmentSizeInBlocks[bus] = snd_pcm_hw_params_get_period_size(hwparams[bus], &dir);
        fragmentSize[bus]=fragmentSizeInBlocks[bus] * channels * (snd_pcm_format_width(SND_PCM_FORMAT_S16_LE) / 8);

        // write the parameters to device
        r = snd_pcm_hw_params(pcm_handle[bus], hwparams[bus]);
        if (r < 0)
                {
                PERROR("Unable to set hw params for playback: %s",snd_strerror(r));
                PEXIT;
                return r;
                }
        PMESG("ALSA BUS # %d Sucessfully open",bus );
        PMESG("     DEV=%s MODE=%s ",cdevice,smode);
        PMESG("     RATE=%d CHANNELS=%d BITDEPTH=%d",rate ,channels,bitDepth);
        PMESG("     FPS=%d  fragmentSizeInBlocks=%d  fragmentSize=%d",fps,(int)fragmentSizeInBlocks[bus],(int)fragmentSize[bus]);
#endif
        busy[bus]=true;        
        PEXIT;
        return 1;
        }



int MustuxAudioDeviceMapper::play_frag(int busNumber, char* data)
        {
#ifdef _ALSA
        int r = snd_pcm_writei(pcm_handle[busNumber], data, fragmentSizeInBlocks[busNumber]);
        if (r < 0) // buffer underrun
                {
                PWARN("Buffer Underrun !");
                //snd_pcm_prepare(pcm_handle[busNumber]);
                return -1;
                }
        /* what the heck is this ?
        if (r == -EAGAIN)
                return 1;
        */

#else
        if (write( deviceId[0] , data , fragmentSize[0] ) == -1)
                {
                PERROR("[OSS Handler] - MustuxAudioDeviceMapper::play_fragment() : cannot write to DSP");
                return -1;
                }
#endif
        return 1;
        }

        
        


int MustuxAudioDeviceMapper::rec_frag(int busNumber, char* data)
        {
#ifdef _ALSA
        //PMESG("Reading "<<fragmentSizeInBlocks[busNumber]<<" blocks ( " << fragmentSize[busNumber] << " bytes)");
        int r = snd_pcm_readi(pcm_handle[busNumber], data, fragmentSizeInBlocks[busNumber]);
        if (r < 0) // buffer underrun
                {
                PWARN("Buffer Underrun !");
                //snd_pcm_prepare(pcm_handle[busNumber]);
                return -1;
                }
        /* what the heck is this ?
        if (r == -EAGAIN)
                return 1;
        */

#else
        if ( read( deviceId[0] , data, fragmentSize[0] ) < 0 )
                {
                PERROR("[OSS Handler] - Cannot read fragment from %s", (const char*) device[0].latin1());
                return -1;
                }
#endif
        return 1;
        }





int MustuxAudioDeviceMapper::get_fragment_size(int bus)
        {
        return fragmentSize[bus];
        }


        
        

int MustuxAudioDeviceMapper::close_device(int bus)
        {
        PENTER;
#ifdef _ALSA
        const char* cdevice = (const char*) device[bus].latin1();
        snd_pcm_close(pcm_handle[bus]); // SHOULDNT IT BE A if (...<0) sentence ?
        PMESG("Sucessfully closed ALSA DEV=%s", cdevice );
#else
        r = close((int)deviceId[0]);
        if (r<0)
                {
                PERROR("[OSS Handler] - Cannot close OSS device %s", (const char*) device[0].latin1());
                PEXIT;
                return -1;
                }
        PMESG("[OSS Handler] Sucessfully closed OSS DEV=%s", (const char*) device[0].latin1() );
#endif
        busy[bus]=false;
        PEXIT;
        return 1;
        }


int MustuxAudioDeviceMapper::get_number_of_buses()
        {
        return nBuses;
        }


QString MustuxAudioDeviceMapper::get_bus_name(int i)
        {
        if ((i>MAX_BUSES) || (i>nBuses) || (i<0))
                return "";
        else
                return busName[i];
        }

QString MustuxAudioDeviceMapper::get_bus_device(int i)
        {
        if ((i>MAX_BUSES) || (i>nBuses) || (i<0))
                return "";
        else
                return device[i];
        }

// eof

